{% extends "admin/base_site.html" %}
{% load i18n %}

{% block content %}
    <form action="" method="post">
        {% csrf_token %}

    <ul>
        {% for e in qs %}
        <li>{{ e.timestamp.date }} {{ e.description }} ({{ e.amount }} {{ e.account.currency }})</li>
        {% endfor %}
    </ul>

    {% trans 'description' %} <input name="description" value="" required />

    {% for obj in qs %}
        <input type="hidden" name="_selected_action" value="{{ obj.pk }}" />
    {% endfor %}

    <input type="hidden" name="action" value="mark_as_manually_settled" />
    <input type="submit" name="apply" value="{% trans 'Mark as manually settled' %}"/>
    </form>
{% endblock %}
