# Generated by Django 2.2.3 on 2019-07-27 16:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('jbank', '0031_auto_20190727_1639'),
    ]

    operations = [
        migrations.AlterField(
            model_name='statementrecorddetail',
            name='unstructured_remittance_info',
            field=models.CharField(blank=True, max_length=2048, verbose_name='unstructured remittance info'),
        ),
    ]
